package casestudies.dna;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Vector;
import java.util.StringTokenizer;
import javax.swing.table.AbstractTableModel;
import javax.swing.text.*;

// for the look and feel
import com.incors.plaf.alloy.*;
import com.incors.plaf.alloy.themes.bedouin.*;


/**
 * GUI to use the pattern match algorithm
 * 1. Insert the main alphabet to search the pattern from into the "Insert Alphabet" field
 * 2. Insert the pattern to search in the alphabet into the "Insert Pattern" field
 * 3. Click Search
 * 4. Results are displayed onto the GUI
 * @author Priya 04/18/05
 * @version 1.0
 */
public class PatternFrame extends JFrame  {
        SearchResults theresults;
        private static String NEWLINE = "\n";
        StyledDocument doc;

        //GUIBUILDER_COMPONENTS:BEGIN (To maintain GUIBuilder readability, do not modify this section.)
        JLabel label1 = new JLabel();
        JTextField alphabet = new JTextField();
        JLabel label2 = new JLabel();
        JTextField pattern = new JTextField();
        JLabel label4 = new JLabel();
        JTextArea matches = new JTextArea();
        JTextPane fancypattern = new JTextPane();
        JButton search = new JButton();
        JButton cancel = new JButton();
        //GUIBUILDER_COMPONENTS:END

        public void guibuilderPositionComponents()
        {
                //GUIBUILDER_POSITIONS:BEGIN (To maintain GUIBuilder readability, do not modify this section.)
                getContentPane().setLayout( new PracticalGridLayout(20,30) );
                getContentPane().add( label1, "2,2,5,2,5,5,5,5" );
                getContentPane().add( alphabet, "2,4,18,2,5,5,5,5" );
                getContentPane().add( label2, "2,6,5,2,5,5,5,5" );
                getContentPane().add( pattern, "2,8,18,2,5,5,5,5" );
                getContentPane().add( label4, "2,10,3,2,5,5,5,5" );
                getContentPane().add(new JScrollPane(matches), "5,10,15,8,5,5,5,5" );
                getContentPane().add(new JScrollPane(fancypattern), "2,18,18,10,5,5,5,5" );
                getContentPane().add( search, "7,28,4,2,5,5,5,5" );
                getContentPane().add( cancel, "11,28,4,2,5,5,5,5" );
                //GUIBUILDER_POSITIONS:END
        }

        public PatternFrame()
        {
                setTitle("Implementation of a Pattern Match Algorithm: Priya Mahajan Math 6790");
                addComponentListener(new ResizeNotice(408,359));
                setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                setLocation(300,300);
                setSize(552,499);
                guibuilderPositionComponents();
                guibuilderComponentProperties();
        }


        public void guibuilderComponentProperties()
        {

                label1.setText("<html><font color=black>Insert Sequence</font></html>");

                alphabet.setText("");

                label2.setText("<html><font color=black>Insert Pattern</font></html>");

                pattern.setText("");

                label4.setText("<html><font color=black>Results:</font></html>");

                matches.setWrapStyleWord(true);
                matches.setLineWrap(true);
                matches.setEditable(false);

                fancypattern.setEditable(false);
                doc = fancypattern.getStyledDocument();
                addStylesToDocument(doc);

                search.setText("Search");
                search.addActionListener(new ActionListener() {
                  public void actionPerformed(ActionEvent e){
                    doSearch();
                  }
                });

                cancel.setText("Cancel");
                cancel.addActionListener(new ActionListener() {
                  public void actionPerformed(ActionEvent e){
                    System.exit(0);
                  }
                });
        }

        protected void addStylesToDocument(StyledDocument doc) {
          //Initialize some styles.
          Style def = StyleContext.getDefaultStyleContext().
              getStyle(StyleContext.DEFAULT_STYLE);

          Style regular = doc.addStyle("regular", def);
          StyleConstants.setFontFamily(def, "SansSerif");

          Style s = doc.addStyle("bold", regular);
          StyleConstants.setBold(s, true);

        }


        private void doSearch() {
          if(alphabet.getText().equals("")) {
            JOptionPane.showMessageDialog(this,
                "Please insert a string into the sequence field",
                "Insert Error",
                JOptionPane.ERROR_MESSAGE);
            return;
          }

          if(pattern.getText().equals("")) {
            JOptionPane.showMessageDialog(this,
                "Please insert a string into the pattern field",
                "Insert Error",
                JOptionPane.ERROR_MESSAGE);
            return;
          }

          try {
            PatternMatchAlgorithm pma = new PatternMatchAlgorithm(
                this.alphabet.getText(),
                this.pattern.getText()
                );
            theresults = pma.startSearch();
            updateUI();
          } catch(Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this, ex.getMessage(),
                                          "Search Error",
                                          JOptionPane.ERROR_MESSAGE);
          }
        } // end doSearch

        private void updateUI() throws Exception {
          // first clear the text area
          matches.setText("");
          fancypattern.setText("");
          int matchSize = theresults.matchesAt.size();

          StringBuffer addtomatch = new StringBuffer();
          if(matchSize == 0)
            addtomatch.append("NO matches found" + NEWLINE);
          else  {
            addtomatch.append("Match at positions:" + NEWLINE);
            addtomatch.append(theresults.matchesAt.toString() + NEWLINE);
          }
          addtomatch.append("Total steps: " + theresults.totalSteps + NEWLINE);
          addtomatch.append("Total number of character comparisons: " + theresults.totalCharComp
                            + NEWLINE);

          // set text in text area
          matches.append(addtomatch.toString());

          // now do for the fancy text area
          int patternLength = this.pattern.getText().length();
          String alphaText = alphabet.getText();
          StringBuffer fancyText = new StringBuffer();
          int earlierMatch=0, currentMatch=0, matchat;
          if(matchSize == 0) {
             doc.insertString(doc.getLength(),alphaText,doc.getStyle("regular"));
          } else {
            for(int i=0; i<matchSize;i++) {
              currentMatch = ((Integer)theresults.matchesAt.get(i)).intValue();
              matchat = currentMatch - earlierMatch;
              String firstStr = alphaText.substring(0, (matchat-1));
              doc.insertString(doc.getLength(),firstStr,doc.getStyle("regular"));
              String thisstr = alphaText.substring((matchat-1), (matchat+patternLength-1));
              doc.insertString(doc.getLength()," " + thisstr + " ",doc.getStyle("bold"));
              alphaText = alphaText.substring((matchat+patternLength-1));
              earlierMatch = currentMatch+(patternLength-1);
            }
              doc.insertString(doc.getLength(),alphaText,doc.getStyle("regular"));
          } // end else
        } // end updateUI


        // Intermediate class between GridBagLayout and GridLayout
        public class PracticalGridLayout implements LayoutManager
        {
            // Find the dimensions of the ideal size
            boolean bStaticMeasurementMade;
            float fStaticWidth;
            float fStaticHeight;
            int nCols;
            int nRows;
            float fCols;
            float fRows;
            float fStaticCellWidth;
            float fStaticCellHeight;
            float fExpandableCellWidth;
            float fExpandableCellHeight;

            // Find if columns and rows are expandable or not.
            boolean [] bColExpandable;
            boolean [] bRowExpandable;
            float fStaticColCount;
            float fStaticRowCount;

            // Store array
            Vector vCoordinates = new Vector();
            Vector vComponents = new Vector();

            // Every time a component is added or removed
            // we must re-evaluate which rows/columns are expandable.
            boolean bExpandabilityEvaluated = false;

            public PracticalGridLayout(int nCols, int nRows)
            {
              bStaticMeasurementMade = false;
              this.nCols = nCols;
              this.nRows = nRows;
              this.fCols = (float) nCols;
              this.fRows = (float) nRows;

              bColExpandable = new boolean[nCols];
              bRowExpandable = new boolean[nRows];
            }

            // See which rows/columns are expandable, and which are static.
            public void evaluateExpandability()
            {
              // Clear arrays.  All columns are static.
              for(int i=0; i< nCols; i++)
              {
                bColExpandable[i] = false;
              }

              // All rows are static.
              for(int i=0; i< nRows; i++)
              {
                bRowExpandable[i] = false;
              }

              // See which rows and columns are expandable
              for (int iCount = 0; iCount < vComponents.size(); iCount++)
              {
                // Get the coordinates
                  Component comp = (Component)vComponents.elementAt(iCount);
                  String strCoordinates = (String)vCoordinates.elementAt(iCount);
                  StringTokenizer st = new StringTokenizer(strCoordinates,",");

                  int iLeft = Integer.valueOf(st.nextToken()).intValue();
                  int iTop = Integer.valueOf(st.nextToken()).intValue();
                  int iWidth = Integer.valueOf(st.nextToken()).intValue();
                  int iHeight = Integer.valueOf(st.nextToken()).intValue();

                  // Some components need to grow as the window does
                  if(comp instanceof JTextArea
                     ||comp instanceof JTree
                     || comp instanceof JTable
                     || comp instanceof JList
                     || comp instanceof JTabbedPane
                     || comp instanceof JScrollPane
                     || comp instanceof JTabbedPane)
                  {
                    // Cols
                    for(int j=iLeft; j <iLeft + iWidth; j++)
                    {
                      bColExpandable[j-1] = true;
                    }

                    // Rows
                    for(int j=iTop; j <iTop + iHeight; j++)
                    {
                      bRowExpandable[j-1] = true;
                    }
                  }
                  else if(comp instanceof JTextField
                     ||comp instanceof JComboBox
                     || comp instanceof JButton)
                  {
                    // Cols
                    for(int j=iLeft; j <iLeft + iWidth; j++)
                    {
                      bColExpandable[j-1] = true;
                    }
                  }

              }

              // We need to know now many cols can expand.
              int nStaticColCount = 0;
              for(int i=0; i< nCols; i++)
              {
                if(bColExpandable[i] == false)
                  nStaticColCount++;
              }
              fStaticColCount = (float) nStaticColCount;

              // We need to know now many rows can expand.
              int nStaticRowCount = 0;
              for(int i=0; i< nRows; i++)
              {
                if(bRowExpandable[i] == false)
                  nStaticRowCount++;
              }
              fStaticRowCount = (float) nStaticRowCount;

              bExpandabilityEvaluated = true;
            }

            // This string describes Where, How Big, and What Insets
            public void addLayoutComponent(String sCoordinates, Component comp)
            {
                vCoordinates.addElement(sCoordinates);
                vComponents.addElement(comp);
                bExpandabilityEvaluated = false;
            }

            // Remove the selected component.
            public void removeLayoutComponent(Component comp)
            {
                int iCount = vComponents.indexOf(comp);
                if (iCount != -1)
                {
                    vCoordinates.removeElementAt(iCount);
                    vComponents.removeElementAt(iCount);
                }
                bExpandabilityEvaluated = false;
            }

            public Dimension preferredLayoutSize(Container target)
            {
                return (new Dimension(10, 10));
            }

            public Dimension minimumLayoutSize(Container target)
            {
                return (new Dimension(10, 10));
            }

            public void layoutContainer(Container target)
            {
              Dimension d = target.getSize();

              // The first time this is seen, set the static width and height.
              if(bStaticMeasurementMade == false)
              {
                fStaticWidth = (float)d.width;
                fStaticHeight = (float)d.height;
                fStaticCellWidth = fStaticWidth/fCols;
                fStaticCellHeight = fStaticHeight/fRows;
                bStaticMeasurementMade = true;
              }

              // If a component has been added or removed, go re-evaluate expandability.
              if(this.bExpandabilityEvaluated == false)
                evaluateExpandability();

              // For the window at this size, how big are expandable column/rows?

              float fCurrentWidth = (float)d.width;
              float fCurrentHeight = (float)d.height;
              fExpandableCellWidth = (fCurrentWidth - fStaticColCount*fStaticCellWidth)/(fCols - fStaticColCount);
              fExpandableCellHeight = (fCurrentHeight - fStaticRowCount*fStaticCellHeight)/(fRows - fStaticRowCount);

              // Add componets to the window.
              for (int iCount = 0; iCount < vComponents.size(); iCount++)
              {
                // Get the coordinates
                Component comp = (Component) vComponents.elementAt(iCount);
                String strCoordinates = (String) vCoordinates.elementAt(iCount);
                StringTokenizer st = new StringTokenizer(strCoordinates, ",");

                int iLeft = Integer.valueOf(st.nextToken()).intValue();
                int iTop = Integer.valueOf(st.nextToken()).intValue();
                int iWidth = Integer.valueOf(st.nextToken()).intValue();
                int iHeight = Integer.valueOf(st.nextToken()).intValue();
                int iInsetLeft = Integer.valueOf(st.nextToken()).intValue();
                int iInsetTop = Integer.valueOf(st.nextToken()).intValue();
                int iInsetRight = Integer.valueOf(st.nextToken()).intValue();
                int iInsetBottom = Integer.valueOf(st.nextToken()).intValue();

                int x1 = findColPosition(iLeft);
                int y1 = findRowPosition(iTop);
                int x2 = findColPosition(iLeft + iWidth);
                int y2 = findRowPosition(iTop + iHeight);

                x1 = x1 + iInsetLeft;
                y1 = y1 + iInsetTop;
                int width = (x2 - iInsetRight) - x1;
                int height = (y2 - iInsetBottom) - y1;

                comp.setBounds(x1, y1, width, height);

              }
              String str;


            }

            // Add up static and expandable rows to find where the
            // component is.
            public int findColPosition(int nEdge)
            {
              nEdge--;
              float dTotal =0;
              for(int i=1; i<= nEdge; i++)
              {
                if(bColExpandable[i-1])
                  dTotal = dTotal + fExpandableCellWidth;
                else
                  dTotal = dTotal + fStaticCellWidth;
              }
              return (int)dTotal;
            }

            // Add up static and expandable rows to find where the
            // component is.
            public int findRowPosition(int nEdge)
            {
              nEdge--;
              float dTotal =0;
              for(int i=1; i<= nEdge; i++)
              {
                if(bRowExpandable[i-1])
                  dTotal = dTotal + fExpandableCellHeight;
                else
                  dTotal = dTotal + fStaticCellHeight;
              }
              return (int)dTotal;
            }



        }


        // Maintain a minimum size
        class ResizeNotice implements ComponentListener
        {
                int nMinimumWidth;
                int nMinimumHeight;

                public ResizeNotice(int nMinimumWidth, int nMinimumHeight)
                {
                        this.nMinimumWidth = nMinimumWidth;
                        this.nMinimumHeight = nMinimumHeight;
                }

                public void componentHidden(ComponentEvent e)
                {}

                public  void componentMoved(ComponentEvent e)
                {}

                public  void componentResized(ComponentEvent e)
                {
                        int nThisWidth = e.getComponent().getWidth();
                        int nThisHeight = e.getComponent().getHeight();

                        if(nThisWidth <= nMinimumWidth)
                                nThisWidth = nMinimumWidth;
                        if(nThisHeight <= nMinimumHeight)
                                nThisHeight = nMinimumHeight;

                        e.getComponent().setSize(nThisWidth,nThisHeight);
                }

                public  void componentShown(ComponentEvent e)
                {}

        }

        public static void main(String[] args)
        {
                try
                {
//                        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    com.incors.plaf.alloy.AlloyLookAndFeel.setProperty("alloy.licenseCode",
                     "e#Thanh_Truong#i9umvy#avij44"); // version 1.4.4
                    AlloyLookAndFeel alloyLnF = new com.incors.plaf.alloy.AlloyLookAndFeel();
                    javax.swing.UIManager.setLookAndFeel(alloyLnF);
                    alloyLnF.setTheme(new BedouinTheme(), true);

                }
                 catch (Exception e)
                {
                        System.out.println("Unable to find System Look and Feel");
                }
                PatternFrame frame = new PatternFrame();
                frame.show();
        }

}
